#version 430

// outputs from vertex shader
in vec3 vert_world_position;
in vec3 vert_normal;
in vec2 vert_uv;
in vec3 light;
in vec3 directionToCamera;

layout(location = 20) uniform sampler2D color_texture;
layout(location = 21) uniform vec4 base_color;
layout(location = 22) uniform float use_texture;
layout(location = 23) uniform float glow;

layout(location = 30) uniform vec3 light_dir;
layout(location = 31) uniform vec3 light_color;
layout(location = 32) uniform vec3 camera_position;

const float ambient = 0.3;
const vec3 specular_color = vec3(1.0, 1.0, 1.0);
const float shininess = 16.0;
const float brightness_threshold = 1.0;
const float bloom_damping = 0.25;

layout(location = 0) out vec4 frag_color;
layout(location = 1) out vec4 glow_color;

void main() {

  vec3 normal = normalize(vert_normal);
  
  float diffuse = max(dot(light_dir, normal), 0.0);
  float specular = 0.0;
  
  if (diffuse > 0.0) {
    vec3 view_dir = normalize(camera_position - vert_world_position);
    vec3 half_dir = normalize(light_dir + view_dir);
    specular = pow(max(dot(half_dir, normal), 0.0), shininess);
  }

  
  vec4 tex_color = texture(color_texture, vert_uv);
  vec3 light = (diffuse * light_color) + (specular * specular_color) + vec3(ambient);
  vec3 color = light * mix(base_color.rgb, tex_color.rgb, min(tex_color.a, use_texture));
  
  // quantize intensity for cel shading, numShades = 3
  float intensity = diffuse + specular + ambient; 
  float shadeIntensity = ceil(intensity * 3)/ 3;
  
  frag_color = vec4(color*shadeIntensity, max(tex_color.a, use_texture));
  vec4 tmp_glow = vec4(base_color.rgb*glow,glow);
  
  float brightness = dot(frag_color.rgb, vec3(0.2126, 0.7152, 0.0722));
  if(brightness > brightness_threshold) {
      tmp_glow = vec4(frag_color.rgb, bloom_damping);
  }
  glow_color = tmp_glow;
}